/* -*- Mode: c++ -*-
 * $Id: s.xotclTrace.c 1.5 01/10/11 18:37:46+02:00 neumann@mohegan.wu-wien.ac.at $
 *  
 *  Extended Object Tcl (XOTcl)

 *  Copyright (C) 1999-2002 Gustaf Neumann, Uwe Zdun
 *
 *  findXOTcl.h -- 
 *  
 *  Find XOTcl library location for package require and put it into the auto_path
 *  
 */

#include "tclInt.h"
#include <tclPort.h>
#include "xotclInt.h"

#define isXOTclDirString(m) (\
	*m   == '/' && m[1] == 'x' && m[2] == 'o' && m[3] == 't' && \
	m[4] == 'c' && m[5] == 'l' && m[6] == '-')

int isLastOccurenceOfXOTclDirString(char* d, int count, int length) {
  while (count < length) {
    if (isXOTclDirString(d)) {
      return 0;
    }
    d++; count++;
  }
  return 1;
}

int checkLibFileExists(Tcl_Interp* in, Tcl_DString* cmdPtr) {
  Tcl_DString lib, fn;
  char *fileName;
  int success;

  Tcl_DStringInit(&fn);
  Tcl_DStringInit(&lib);
  Tcl_DStringAppend(&lib, 
		    Tcl_DStringValue(cmdPtr), Tcl_DStringLength(cmdPtr));

# if defined(__WIN32__)
  Tcl_DStringAppend(&lib, "/win/Release/xotcl", -1);
  Tcl_DStringAppend(&lib, XOTCLVERSION, -1);
# else
  Tcl_DStringAppend(&lib, "/unix/libxotcl", -1);
  Tcl_DStringAppend(&lib, VERSION, -1);
# endif

  Tcl_DStringAppend(&lib, TCL_SHLIB_EXT, -1);

  fileName = Tcl_TranslateFileName(in, Tcl_DStringValue(&lib), &fn);
  if (fileName) {
#ifdef PRE81
    success = (TclAccess(fileName, R_OK) == 0);
#else
    success = (Tcl_Access(fileName, R_OK) == 0);
#endif
    Tcl_DStringFree(&fn);
  } else {
    success = 0;
  }
  Tcl_DStringFree(&lib);
  return success;
}

int XOTclCorrectAutoPath(Tcl_Interp* in) {
  Tcl_DString ds, topDir, version, cmd;
  char  *dir, *d;
  int count = 0;
  int dsLength;

#ifdef PRE81
  d = TclGetCwd(in);
  if (d == NULL) {
    return TCL_ERROR;
  }
  Tcl_DStringInit(&ds);
  Tcl_DStringAppend(&ds, d, -1);
#else
  if (Tcl_GetCwd(in, &ds) == NULL) {
    return TCL_ERROR;
  }
#endif

  dsLength = Tcl_DStringLength(&ds);
  d = dir =  Tcl_DStringValue(&ds);

  Tcl_DStringInit(&cmd);

  while (count < dsLength) {
    if (isXOTclDirString(d) &&
	isLastOccurenceOfXOTclDirString(d+6, count+6, dsLength)) {
      /* which version number ?*/
      count+=7;
      d+=7;
      /* search next '/', if any 
	 under win "while (d && *d != '/')" does not work,
	 therefore we use the length
      */
      while (count < dsLength && *d != '/') {
	count++; d++;
      }
      Tcl_DStringAppend(&cmd, dir, count);
      /*fprintf(stderr, "XOTCL DIR is %s\n", Tcl_DStringValue(&cmd));*/
      /* 
       * check whether we have the shared library in place 
       * otherwise we deliver an error
       */
      if (checkLibFileExists(in, &cmd) == 0) {
	Tcl_DStringFree(&cmd);
	Tcl_DStringInit(&cmd);
      } else {
	Tcl_DStringAppend(&cmd, " ", 1);
	/*fprintf(stderr, "Library is there %s\n", Tcl_DStringValue(&cmd));*/
      }
      break;
    }
    count++; d++;
  }

  Tcl_DStringAppend(&cmd, XOLIBPKG, -1);
  Tcl_DStringAppend(&cmd, " ", 1);

  Tcl_DStringAppend(&cmd, Tcl_GetVar(in, "auto_path", TCL_GLOBAL_ONLY), -1);
  Tcl_SetVar(in, "auto_path", Tcl_DStringValue(&cmd), TCL_GLOBAL_ONLY);

  /*fprintf(stderr, "auto_path in findXOTcl= %s\n", Tcl_DStringValue(&cmd));*/

  Tcl_DStringFree(&cmd);
  Tcl_DStringFree(&ds);
  return TCL_OK;
}
